﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.AutoCAD.Colors

<Assembly: CommandClass(GetType(dotNetTraining_Webcast_Demos.Webcast_3_Demos))> 

Namespace dotNetTraining_Webcast_Demos

    Class Webcast_3_Demos

        ' Add a line to model and to paperspace
        <CommandMethod("demoAddLines")> _
        Public Shared Sub addLinesDemo()
            'Public Sub addLinesDemo() ' Added Shared for Lab8, running from a context menu
            ' get the editor
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ' get database
            Dim db As Database = ed.Document.Database

            ' start a transaction
            Dim trans As Transaction = db.TransactionManager.StartTransaction()


            Try
                ' creating a new line
                Dim line1 As New Line(New Point3d(0, 0, 0), New Point3d(10, 10, 0))
                ' creat another line
                Dim line2 As New Line(New Point3d(0, 0, 0), New Point3d(14, 7, 0))
                ' get block table
                Dim blkTable As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)
                ' get model space
                Dim btrModelSpace As BlockTableRecord = _
                    trans.GetObject(blkTable(BlockTableRecord.ModelSpace), OpenMode.ForWrite)
                ' Get paper space
                Dim btrPaperSpace As BlockTableRecord = _
                    trans.GetObject(blkTable(BlockTableRecord.PaperSpace), OpenMode.ForWrite)
                ' append the line
                btrModelSpace.AppendEntity(line1)
                ' append the second line to paper space
                btrPaperSpace.AppendEntity(line2)
                ' add entity to transaction
                trans.AddNewlyCreatedDBObject(line1, True)

                trans.AddNewlyCreatedDBObject(line2, True)
                ' Commit transaction
                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            Finally

                trans.Dispose()

            End Try


        End Sub

        ' add a layer and text to model space
        <CommandMethod("demoAddLayer")> _
        Public Sub addLayerAndTextDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim db As Database = ed.Document.Database

            Dim trans As Transaction = db.TransactionManager.StartTransaction()

            ' declare a LayerTableRecord
            Dim layer1 As LayerTableRecord

            Try
                ' Layer Table
                Dim layerTable As LayerTable = trans.GetObject(db.LayerTableId, OpenMode.ForRead)

                ' see if testLayer exists
                If layerTable.Has("testLayer") Then
                    ' tell the user
                    ed.WriteMessage("testLayer already exists" & vbCrLf)
                    ' instantiate the LayerTableRecord 
                    layer1 = trans.GetObject(layerTable("testLayer"), OpenMode.ForWrite)
                Else
                    ' Upgrade the open of the layer table
                    layerTable.UpgradeOpen()
                    ' create new layer table record
                    layer1 = New LayerTableRecord()

                    layer1.Name = "testLayer"
                    ' set the color
                    layer1.Color = Color.FromColorIndex(ColorMethod.ByAci, 1)

                    ' add layer table record
                    layerTable.Add(layer1)
                    ' add to transaction
                    trans.AddNewlyCreatedDBObject(layer1, True)

                End If

                ' create text
                Dim text1 As DBText = New DBText()
                ' add the text value
                text1.TextString = "Some New Text"
                ' set the layer for the text
                text1.LayerId = layer1.Id
                ' set the position
                text1.Position = New Point3d(7, 7, 0)
                ' set default settings for text
                text1.SetDatabaseDefaults()
                ' get the block table
                Dim blkTable As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)
                ' get modelspace
                Dim btrModelSpace As BlockTableRecord = _
                       trans.GetObject(blkTable(BlockTableRecord.ModelSpace), OpenMode.ForWrite)
                ' append text to model space
                btrModelSpace.AppendEntity(text1)
                ' add to transaction
                trans.AddNewlyCreatedDBObject(text1, True)
                ' commit transaction
                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            Finally

                trans.Dispose()

            End Try

        End Sub

        ' add a block
        <CommandMethod("demoAddBlock")> _
        Public Sub addBlockDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim db As Database = ed.Document.Database

            Dim trans As Transaction = db.TransactionManager.StartTransaction()


            Try
                Dim blkTable As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)

                ' check to see if the block table has our block
                If blkTable.Has("testBlock") Then

                    ed.WriteMessage("testBlock already exists" & vbCrLf)

                    Return

                Else

                    ' upgrade the block table for write
                    blkTable.UpgradeOpen()

                    ' create a new blocktable record
                    Dim blkTableRec As BlockTableRecord = New BlockTableRecord()
                    ' give it a name
                    blkTableRec.Name = "testBlock"
                    ' add the block table record
                    blkTable.Add(blkTableRec)
                    ' add to transaction
                    trans.AddNewlyCreatedDBObject(blkTableRec, True)

                    Dim center As Point3d = New Point3d(5.0, 5.0, 0.0)
                    Dim normal As Vector3d = New Vector3d(0, 0, 1)
                    Dim majorAxis As Vector3d = New Vector3d(20, 0, 0) ' the length of major axis
                    Dim radiusRatio As Double = 0.5 ' <=1
                    Dim startAng As Double = 0  ' In radian
                    Dim endAng As Double = 360 * Math.Atan(1) / 45 ' in radian
                    ' create an ellipse
                    Dim ellipse1 As Ellipse = _
                        New Ellipse(center, normal, majorAxis, radiusRatio, startAng, endAng)
                    ' set the default settins
                    ellipse1.SetDatabaseDefaults()

                    ' append the ellipse to our block table record
                    blkTableRec.AppendEntity(ellipse1)
                    ' add ellipse to transaction
                    trans.AddNewlyCreatedDBObject(ellipse1, True)
                End If

                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            Finally

                trans.Dispose()

            End Try

        End Sub

        ' add a block reference to model space
        <CommandMethod("demoAddBlockRef")> _
        Public Sub addBlockReferenceDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim db As Database = ed.Document.Database

            Dim trans As Transaction = db.TransactionManager.StartTransaction()


            Try

                Dim blkTable As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)
                ' make sure our block exists
                If blkTable.Has("testBlock") Then
                    ' get the objectId of the blocktablerecord
                    Dim objId As ObjectId = blkTable("testBlock")
                    ' create a BlockReference
                    Dim blockRef As BlockReference = New BlockReference(New Point3d(0, 0, 0), objId)

                    ' Get model space
                    Dim btrModelSpace As BlockTableRecord = _
                        trans.GetObject(blkTable(BlockTableRecord.ModelSpace), OpenMode.ForWrite)
                    ' put the BlockReference in model space
                    btrModelSpace.AppendEntity(blockRef)
                    ' add the blockReference to transaction
                    trans.AddNewlyCreatedDBObject(blockRef, True)


                Else

                    ed.WriteMessage("testBlock does NOT exists" & vbCrLf)


                End If

                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            Finally

                trans.Dispose()

            End Try

        End Sub


        ' Create new drawing
        <CommandMethod("demoCreateNewDwg", CommandFlags.Session)> _
        Public Sub createNewDwgDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Try
                ' create new database
                Dim db As New Database(True, True)
                ' save dwg
                db.SaveAs("c:\temp\a_demoInWebcast.dwg", DwgVersion.Newest)
                ' close the file
                db.CloseInput(True)
            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            End Try

        End Sub

    End Class

End Namespace
