﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

<Assembly: CommandClass(GetType(dotNetTraining_Webcast_Demos.Webcast_7_Demos))> 


Namespace dotNetTraining_Webcast_Demos

    Public Class Webcast_7_Demos
        <CommandMethod("DemoBlockRefJig")> _
        Public Shared Sub BlockRefJigDemo()

            Dim pStringOpts As New PromptStringOptions(vbLf & "Enter Block Name")

            Dim pResBlkName As PromptResult = Application.DocumentManager.MdiActiveDocument.Editor.GetString(pStringOpts)


            If pResBlkName.Status <> PromptStatus.OK Then
                Return
            End If


            pStringOpts.Message = "Enter Value for the Attributes"


            pStringOpts.AllowSpaces = True

            Dim pResAttName As PromptResult = Application.DocumentManager.MdiActiveDocument.Editor.GetString(pStringOpts)

            If pResAttName.Status <> PromptStatus.OK Then
                Return
            End If


            Using trans As Transaction = Application.DocumentManager.MdiActiveDocument.TransactionManager.StartTransaction()

                Dim bt As BlockTable = _
                    CType(trans.GetObject(HostApplicationServices.WorkingDatabase.BlockTableId, OpenMode.ForRead), BlockTable)

                Dim currentSpace As BlockTableRecord = _
                   CType(Application.DocumentManager.MdiActiveDocument.Database.CurrentSpaceId.GetObject(OpenMode.ForWrite), BlockTableRecord)
                ' Ensure that the Block exists
                If Not bt.Has(pResBlkName.StringResult) Then
                    Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage _
                        (vbLf & "Block """ + pResBlkName.StringResult & """ does NOT exist..." & vbLf)
                    Return
                End If


                Dim btr As BlockTableRecord = _
                    CType(trans.GetObject(bt(pResBlkName.StringResult), OpenMode.ForWrite), BlockTableRecord)

                'Block needs to be inserted to current space before being able to 
                'append attribute to it
                Dim blkRef As New BlockReference(New Point3d(), btr.ObjectId)

                currentSpace.AppendEntity(blkRef)
                trans.AddNewlyCreatedDBObject(blkRef, True)
                ' see the Block has attributes 
                If btr.HasAttributeDefinitions Then
                    Dim count As Integer = 0

                    For Each blkEntId As ObjectId In btr
                        Dim AttributeDef As AttributeDefinition = _
                            TryCast(trans.GetObject(blkEntId, OpenMode.ForRead), AttributeDefinition)

                        If AttributeDef IsNot Nothing AndAlso Not AttributeDef.Constant Then
                            count += 1
                            ' create a new Att Ref
                            Dim AttributeRef As New AttributeReference()
                            ' set the setting Att Def
                            AttributeRef.SetAttributeFromBlock(AttributeDef, blkRef.BlockTransform)
                            ' set the positionof att ref 
                            AttributeRef.Position = AttributeDef.Position.TransformBy(blkRef.BlockTransform)
                            ' make the text for att ref
                            AttributeRef.TextString = pResAttName.StringResult & count.ToString()
                            ' append the att ref
                            blkRef.AttributeCollection.AppendAttribute(AttributeRef)
                            ' tell the transaction about the Att Ref
                            trans.AddNewlyCreatedDBObject(AttributeRef, True)
                        End If
                    Next
                End If



                Dim myJig As New CBlockJig(blkRef)

                Dim promptResult As PromptResult = _
                    Application.DocumentManager.MdiActiveDocument.Editor.Drag(myJig)

                If promptResult.Status <> PromptStatus.OK Then

                    Return

                End If

                'Commit changes if user returned ok, otherwise discard
                trans.Commit()
            End Using
        End Sub


    End Class


    ' Create class that inherits from entityJig
    Class CBlockJig
        Inherits EntityJig
        ' global variable for a point
        Private mPosition As Point3d
        ' global variable for a dictionary
        Private mAttributePosDictionary As Dictionary(Of String, Point3d)

        ' constructor
        Public Sub New(ByVal blkRef As BlockReference)
            ' compile error occurs if not called
            MyBase.New(blkRef)
            ' change the global point variable
            mPosition = blkRef.Position

            ' instantiate the dictionary
            mAttributePosDictionary = New Dictionary(Of String, Point3d)()

            ' get the Block
            Dim btr As BlockTableRecord = _
                CType(blkRef.BlockTableRecord.GetObject(OpenMode.ForRead), BlockTableRecord)

            ' see if the block has attribute
            If btr.HasAttributeDefinitions Then
                ' go through each ObjectId in the block
                For Each blkEntId As ObjectId In btr
                    ' get the AttributeDefintion
                    Dim AttDef As AttributeDefinition = _
                        TryCast(blkEntId.GetObject(OpenMode.ForRead), AttributeDefinition)
                    ' see if the variable is not nothing
                    If AttDef IsNot Nothing Then
                        ' Add the tag of the att definition, the Point3d of the attribute
                        mAttributePosDictionary.Add(AttDef.Tag, AttDef.Position)

                    End If

                Next

            End If
        End Sub

        Protected Overrides Function Update() As Boolean
            ' get a Block Ref 
            Dim blkRef As BlockReference = Entity
            ' change the position of the jigged Block Ref
            blkRef.Position = mPosition

            ' Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(" In UPDATE " & mPosition.ToString)

            ' see if the Block Ref has attributes
            If blkRef.AttributeCollection.Count <> 0 Then
                ' go through the attributes, using the ObjectId
                For Each AttributeId As ObjectId In blkRef.AttributeCollection
                    ' Get an Att Ref
                    Dim AttributeRef As AttributeReference = _
                        TryCast(AttributeId.GetObject(OpenMode.ForWrite), AttributeReference)
                    ' change the position of the att ref
                    AttributeRef.Position = _
                        mAttributePosDictionary(AttributeRef.Tag).TransformBy(blkRef.BlockTransform)

                Next
            End If

            'Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage("IN UPDATE")

            Return True
        End Function


        Protected Overrides Function Sampler(ByVal prompts As JigPrompts) As SamplerStatus
            ' create a new JigPromptPointOptions.
            Dim jigPrPointOpts As New JigPromptPointOptions()
            ' ensure that no zero is entered
            jigPrPointOpts.UserInputControls = UserInputControls.NoZeroResponseAccepted

            ' get the point, 
            Dim pPointRes As PromptPointResult = prompts.AcquirePoint(jigPrPointOpts)

            ' test if the point has change
            If mPosition = pPointRes.Value Then

                'Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(" IN SAMPLER Returning No Change ")

                Return SamplerStatus.NoChange


            End If
            ' update the global point varible
            mPosition = pPointRes.Value

            ' Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(" IN SAMPLER returning OK ")

            Return SamplerStatus.OK

        End Function

    End Class
   

End Namespace
