﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput
' another import
Imports Autodesk.AutoCAD.Windows
' import for 
Imports System.Runtime.InteropServices

' command class attribute
<Assembly: CommandClass(GetType(dotNetTraining_Webcast_Demos.Webcast_8_Demos))> 

<Assembly: ExtensionApplication(GetType(dotNetTraining_Webcast_Demos.Webcast_8_Demos))> 


Namespace dotNetTraining_Webcast_Demos

    Public Class Webcast_8_Demos
        Implements IExtensionApplication
        ' global vars for our menu
        Dim m_ContextMenu As ContextMenuExtension

        Dim m_ObjectMenu As ContextMenuExtension

        Public Shared m_DraftingSetting As String = "Some Default Value"


        <DllImport("acad.exe", CharSet:=CharSet.Unicode, EntryPoint:="?acedPostCommand@@YAHPB_W@Z")> _
        Public Shared Function acedPostCommand(ByVal cmd As String) As Boolean
        End Function

        ' gets called when the dll is loaded
        Public Sub Initialize() Implements Autodesk.AutoCAD.Runtime.IExtensionApplication.Initialize
            ' call our function
            AddContextMenu()
            ' add the object menus
            AddObject_RightClickMenu()

            AddTabToDraftingSettings()

        End Sub

        Public Sub Terminate() Implements Autodesk.AutoCAD.Runtime.IExtensionApplication.Terminate
            RemoveContextMenus()
        End Sub


        Private Sub AddContextMenu()
            ' get the editor
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Try
                ' instantiate global var
                m_ContextMenu = New ContextMenuExtension
                ' set the text the user will see on the menu
                m_ContextMenu.Title = "A Custom Menu"
                ' pop out menu
                Dim mnuItem As MenuItem = New MenuItem("Run DemoBlockRefJig")
                ' add the event for the menu click
                AddHandler mnuItem.Click, AddressOf runBlockRefJigDemo_WhenClicked
                ' add the menu item
                m_ContextMenu.MenuItems.Add(mnuItem)
                ' add the context menu, passing contextmenu extension
                Application.AddDefaultContextMenuExtension(m_ContextMenu)

            Catch ex As System.Exception

                ed.WriteMessage("Error Adding Context Menu: " + ex.Message)

            End Try

        End Sub

        Private Sub AddObject_RightClickMenu()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Try

                m_ObjectMenu = New ContextMenuExtension

                m_ObjectMenu.Title = "An Object Menu"

                Dim mnuItem As MenuItem = New MenuItem("Run addLinesDemo")

                AddHandler mnuItem.Click, AddressOf run_addLinesDemo_WhenClicked

                m_ObjectMenu.MenuItems.Add(mnuItem)
                ' create an rxclass
                Dim circleClass As RXClass = RXObject.GetClass(GetType(Circle))
                ' adding the object menu
                Application.AddObjectContextMenuExtension(circleClass, m_ObjectMenu)


            Catch ex As System.Exception

                ed.WriteMessage("Error Adding Object Menu: " + ex.Message)

            End Try

        End Sub

        Public Sub runBlockRefJigDemo_WhenClicked()

            Using docLock As DocumentLock = _
             Application.DocumentManager.MdiActiveDocument.LockDocument

                Webcast_7_Demos.BlockRefJigDemo()

                acedPostCommand("CANCELCMD")

            End Using
        End Sub

        Public Sub run_addLinesDemo_WhenClicked()
            Using docLock As DocumentLock = _
                Application.DocumentManager.MdiActiveDocument.LockDocument()

                Webcast_3_Demos.addLinesDemo()

            End Using
        End Sub


        Public Sub RemoveContextMenus()

            Try

                If Not m_ContextMenu Is Nothing Then
                    Application.RemoveDefaultContextMenuExtension(m_ContextMenu)
                    m_ContextMenu = Nothing
                End If

                If Not m_ObjectMenu Is Nothing Then
                    Dim circleClass As RXClass = RXObject.GetClass(GetType(Circle))
                    Application.RemoveObjectContextMenuExtension(circleClass, m_ObjectMenu)
                    m_ObjectMenu = Nothing
                End If


            Catch ex As System.Exception

                System.Diagnostics.Debug.Print(ex.ToString)

            End Try
        End Sub

        Public Shared Sub AddTabToDraftingSettings()
            ' add the event when DSETTINGS is run
            AddHandler Application.DisplayingDraftingSettingsDialog, _
                   AddressOf myDraftingSettingsTabHandler

        End Sub


        Private Shared Sub myDraftingSettingsTabHandler(ByVal sender As Object, _
                         ByVal e As Autodesk.AutoCAD.ApplicationServices.TabbedDialogEventArgs)

            ' create a new control
            Dim draftingSettingsCtrl As draftingSettingsTabCtrl = New draftingSettingsTabCtrl

            ' create the action
            Dim tabbedDialogAct As TabbedDialogAction = _
                      New TabbedDialogAction(AddressOf draftingSettingsCtrl.OnOk)

            ' create the tabbeddialogextenstion
            Dim tabbedDialogExt As TabbedDialogExtension = _
                         New TabbedDialogExtension(draftingSettingsCtrl, tabbedDialogAct)

            ' add the control to the DSETTINGS dialog
            e.AddTab("Custom settings", tabbedDialogExt)

        End Sub

        <CommandMethod("DemoDisplayCustomDraftingSetting")> _
        Public Sub getCustomDraftingSettingDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor
            ed.WriteMessage(m_DraftingSetting.ToString())

        End Sub

    End Class

End Namespace
